@@ -111,6 +111,12 @@ def _generator_def(nodes):
         yield prepend_newline * '\n' + _generate_def_visitor[type(node)](node) + '\n'
         last_node = node
 
+def _member_access_statement(member):
+    out = '&payload->%s' % member.name
+    if isinstance(member, model.StructMember) and member.array:
+        out = out[1:]
+    return out
+
 def _generate_swap_struct(struct):
     def gen_member(member, delimiters = []):
         if member.array:
@@ -120,24 +126,23 @@ def _generate_swap_struct(struct):
                 bound = member.bound
                 if member.bound not in delimiters:
                     bound = 'payload->' + bound
-                return 'swap_n_{0}(payload->{1}, {2})'.format(
-                    swap_mode, member.name, bound)
+                return 'swap_n_{0}({1}, {2})'.format(
+                    swap_mode, _member_access_statement(member), bound)
             elif not member.bound and member.size:
-                return 'swap_n_{0}(payload->{1}, {2})'.format(
-                    swap_mode, member.name, member.size)
+                return 'swap_n_{0}({1}, {2})'.format(
+                    swap_mode, _member_access_statement(member), member.size)
         else:
             if member.optional:
                 preamble = 'swap(&payload->has_{0});\nif (payload->has_{0}) '.format(member.name)
             else:
                 preamble = ''
-            return preamble + 'swap(&payload->{0})'.format(member.name)
+            return preamble + 'swap({0})'.format(_member_access_statement(member))
 
     def gen_last_member(name, last_mem, delimiters = []):
         if last_mem.kind == model.Kind.UNLIMITED or last_mem.greedy:
-            return 'return cast<{0}*>({1}payload->{2});\n'.format(
+            return 'return cast<{0}*>({1});\n'.format(
                 name,
-                '' if last_mem.array else '&',
-                last_mem.name
+                _member_access_statement(last_mem)
             )
         elif last_mem.kind == model.Kind.DYNAMIC or last_mem.dynamic:
             return 'return cast<{0}*>({1});\n'.format(
@@ -209,7 +214,9 @@ def _generate_swap_union(union):
             '    return payload + 1;\n'
             '}}\n').format(
                 union.name,
-                ''.join(8 * ' ' + 'case {0}::discriminator_{1}: swap(&payload->{1}); break;\n'.format(union.name, m.name) for m in union.members)
+                ''.join(8 * ' ' + 'case {0}::discriminator_{1}: swap({2}); break;\n'.format(
+                    union.name, m.name, _member_access_statement(m)
+                ) for m in union.members)
             )
 
 _generate_swap_visitor = {
@@ -251,6 +258,26 @@ class CppGenerator(object):
     def generate_definitions(self, nodes):
         return ''.join(_generator_def(nodes))
 
+    def generate_swap_declarations(self, nodes):
+        out = ''.join(
+            'template <> inline {0}* swap<{0}>({0}* in) {{ swap(reinterpret_cast<uint32_t*>(in)); return in + 1; }}\n'
+            .format(node.name)
+            for node in nodes
+            if isinstance(node, (model.Enum))
+        )
+        out += ''.join(
+            'template <> {0}* swap<{0}>({0}*);\n'.format(node.name)
+            for node in nodes
+            if isinstance(node, (model.Struct, model.Union))
+        )
+        return out.join((
+            'namespace prophy\n'
+            '{\n'
+            '\n',
+            '\n'
+            '} // namespace prophy\n'
+        ))
+
     def generate_swap(self, nodes):
         return '\n'.join(_generator_swap(nodes))
 
@@ -258,12 +285,15 @@ class CppGenerator(object):
         return '\n'.join((
             header.format(basename),
             self.generate_definitions(nodes),
+            self.generate_swap_declarations(nodes),
             footer.format(basename)
         ))
 
     def serialize_string_cpp(self, nodes, basename):
         return '\n'.join((
+            '#include <prophy/detail/prophy.hpp>\n',
             '#include "{0}.pp.hpp"\n'.format(basename),
+            'using namespace prophy::detail;\n',
             swap_header,
             self.generate_swap(nodes),
             swap_footer
